// Commands.cpp : Implementierungsdatei
//

#include "stdafx.h"
#include "mkdocwrt.h"
#include "Commands.h"
#include "DSWFilesDlg.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCommands

CCommands::CCommands()
{
	m_pApplication = NULL;
	m_pApplicationEventsObj = NULL;
	m_pDebuggerEventsObj = NULL;
}

CCommands::~CCommands()
{
	ASSERT (m_pApplication != NULL);
	m_pApplication->Release();
}

void CCommands::SetApplicationObject(IApplication* pApplication)
{
	// Diese Funktion geht davon aus, dass AddRef bereits auf pApplication angewendet wurde,
	//  was CDSAddIn durch den Aufruf von QueryInterface direkt vor dem Aufruf dieser
	//  Funktion bereits erledigt hat.
	m_pApplication = pApplication;

	// Ereignis-Handler fr Anwendung erzeugen
	XApplicationEventsObj::CreateInstance(&m_pApplicationEventsObj);
	m_pApplicationEventsObj->AddRef();
	m_pApplicationEventsObj->Connect(m_pApplication);
	m_pApplicationEventsObj->m_pCommands = this;

	// Ereignis-Handler fr Debugger erzeugen
	CComPtr<IDispatch> pDebugger;
	if (SUCCEEDED(m_pApplication->get_Debugger(&pDebugger)) 
		&& pDebugger != NULL)
	{
		XDebuggerEventsObj::CreateInstance(&m_pDebuggerEventsObj);
		m_pDebuggerEventsObj->AddRef();
		m_pDebuggerEventsObj->Connect(pDebugger);
		m_pDebuggerEventsObj->m_pCommands = this;
	}
}

void CCommands::UnadviseFromEvents()
{
	ASSERT (m_pApplicationEventsObj != NULL);
	m_pApplicationEventsObj->Disconnect(m_pApplication);
	m_pApplicationEventsObj->Release();
	m_pApplicationEventsObj = NULL;

	if (m_pDebuggerEventsObj != NULL)
	{
		// Da wir die Verbindung zu den Debugger-Ereignissen herstellen konnten, 
		//  sollte es auch mglich sein, erneut auf das Debugger-Objekt zuzugreifen, 
		//  um die Verbindung zu seinen Ereignissen zu trennen (daher das VERIFY_OK weiter unten -- siehe stdafx.h).
		CComPtr<IDispatch> pDebugger;
		VERIFY_OK(m_pApplication->get_Debugger(&pDebugger));
		ASSERT (pDebugger != NULL);
		m_pDebuggerEventsObj->Disconnect(pDebugger);
		m_pDebuggerEventsObj->Release();
		m_pDebuggerEventsObj = NULL;
	}
}


/////////////////////////////////////////////////////////////////////////////
// Ereignis-Handler

// ZU ERLEDIGEN: Fllen Sie die Implementierung fr die Ereignisse aus, die Sie behandeln wollen
//  Verwenden Sie m_pCommands->GetApplicationObject(), um auf das Objekt
//  "Developer Studio Application" zuzugreifen

//Application-Ereignisse

HRESULT CCommands::XApplicationEvents::BeforeBuildStart()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BuildFinish(long /*nNumErrors*/, long /*nNumWarnings*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BeforeApplicationShutDown()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::DocumentOpen(IDispatch* /*theDocument*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::BeforeDocumentClose(IDispatch* /*theDocument*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::DocumentSave(IDispatch* /*theDocument*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewDocument(IDispatch* /*theDocument*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WindowActivate(IDispatch* /*theWindow*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WindowDeactivate(IDispatch* /*theWindow*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceOpen()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::WorkspaceClose()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

HRESULT CCommands::XApplicationEvents::NewWorkspace()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}

// Debugger-Ereignis

HRESULT CCommands::XDebuggerEvents::BreakpointHit(IDispatch* /*pBreakpoint*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return S_OK;
}




void ShowError(LPCTSTR szCaption, DWORD dwError)
{
    LPTSTR lpMsgBuf;
    FormatMessage( 
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM | 
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
        (LPTSTR) &lpMsgBuf,
        0,
        NULL 
    );

    // Process any inserts in lpMsgBuf.
    // ...
    // Display the string.
    MessageBox(::GetActiveWindow(), lpMsgBuf?lpMsgBuf:_T("N/A"), szCaption, MB_OK | MB_ICONINFORMATION );
    // Free the buffer.
    LocalFree( lpMsgBuf );
}


/////////////////////////////////////////////////////////////////////////////
// Ccommands-Methoden


void RemoveReadOnly(LPCTSTR szFile, IApplication* pApplication)
{
    ASSERT(pApplication);
    DWORD dwAtt = GetFileAttributes(szFile);
    if(INVALID_FILE_ATTRIBUTES!=dwAtt)
    {
        if(dwAtt&FILE_ATTRIBUTE_READONLY)
        {
            dwAtt &= ~FILE_ATTRIBUTE_READONLY;
            if(!SetFileAttributes(szFile, dwAtt))
            {
                DWORD dwLastError = GetLastError();
	            VERIFY_OK(pApplication->EnableModeless(VARIANT_FALSE));
                ShowError(_T("Failed!"), dwLastError);
	            VERIFY_OK(pApplication->EnableModeless(VARIANT_TRUE));
            }
        }
    }    
}

STDMETHODIMP CCommands::MkdocwrtCommandMethod() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

    ASSERT(m_pApplication);

	IDispatch* pDispatch = NULL;

	m_pApplication->get_ActiveDocument(&pDispatch);

    if(pDispatch)
    {
	    CComQIPtr<ITextDocument, &IID_ITextDocument> pDocument(pDispatch);
        if(pDocument)
        {
            BSTR strPath;
            HRESULT hRes = pDocument->get_FullName(&strPath);
            if(SUCCEEDED(hRes))
            {
                CString strFile = strPath;
                RemoveReadOnly(strFile, m_pApplication);
            }
        }
        
        pDispatch->Release();
    }

	return S_OK;
}


STDMETHODIMP CCommands::MkdspwrtCommandMethod() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

    ASSERT(m_pApplication);

	IDispatch* pDispatch = NULL;

    m_pApplication->get_ActiveProject(&pDispatch);

    
    if(pDispatch)
    {
	    CComQIPtr<IGenericProject, &IID_IGenericProject> pProject(pDispatch);
        if(pProject)
        {
            BSTR strPath;
            HRESULT hRes = pProject->get_FullName(&strPath);
            if(SUCCEEDED(hRes))
            {
                CString strFile = strPath;
                RemoveReadOnly(strFile, m_pApplication);
            }
        }

        pDispatch->Release();
    }


	return S_OK;
}


#define STARTDOTDSW _T("*.dsw")
#define dimof(a) (sizeof(a)/sizeof(a[0]))

STDMETHODIMP CCommands::MkdswwrtCommandMethod() 
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

    ASSERT(m_pApplication);
    BSTR strPath;

    HRESULT hRes = m_pApplication->get_CurrentDirectory(&strPath);
    if(SUCCEEDED(hRes))
    {
        /// 
        /// we now have the path to the potential dsw files 
        /// in strPath, but we don't know the name of the dsw
        /// file, therefore we enumerate all dsw files now:
        /// 
        CStringList slDswFiles;

        CString strWildcard = strPath;
        if (strWildcard.GetAt(strWildcard.GetLength()-1)!=_T('\\'))
            strWildcard += _T("\\") STARTDOTDSW;
        else
            strWildcard += STARTDOTDSW;

        WIN32_FIND_DATA wfd;
        HANDLE hFind = FindFirstFile(strWildcard, &wfd);
        strWildcard = strWildcard.Left(strWildcard.GetLength() - (dimof(STARTDOTDSW)-1));
        if(INVALID_HANDLE_VALUE!=hFind)
        {
            do 
            {
                if(!(wfd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY))
                {
                    CString strFile = strWildcard + wfd.cFileName;
                    slDswFiles.AddTail(strFile);
                }
            }
            while (FindNextFile(hFind, &wfd));

            VERIFY(FindClose(hFind));
        }

        if(slDswFiles.IsEmpty())
            AfxMessageBox(IDS_NO_DSWFILES_FOUND);
        else
        {
            if(slDswFiles.GetCount()==1)
            {
                /// 
                /// there is only one file in our directory, 
                /// remove the read-only bit from it:
                /// 
                RemoveReadOnly(slDswFiles.GetHead(), m_pApplication);
            }
            else
            {
                /// 
                /// there are multiple dsw files in our directory.
                /// Give the user a choice:
                /// 
                CWnd * pParent = CWnd::FromHandle(::GetActiveWindow());
                ASSERT(pParent);
                CDSWFilesDlg dlg(pParent, &slDswFiles);
                if(IDOK==dlg.DoModal())
                {
                    while (!slDswFiles.IsEmpty())
                        RemoveReadOnly(slDswFiles.RemoveHead(), m_pApplication);
                }
            }
        }
    }

	return S_OK;
}
