// AddInMod.cpp : Implementierungsdatei
//

#include "stdafx.h"
#include "mkdocwrt.h"
#include "DSAddIn.h"
#include "Commands.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



typedef struct tagCMD_SET
{
    LPCTSTR szCmdName;
    UINT uiCmdLocalizedId;
    LPCTSTR szCmdMethod;
}
CMD_SET, *PCMD_SET;

const CMD_SET g_cmdSet[] =
{
    {_T("MkdocwrtCommand"), IDS_CMD_STRING, _T("MkdocwrtCommandMethod")},
    {_T("MkdspwrtCommand"), IDS_CMD_MKDSPWRITABLE, _T("MkdspwrtCommandMethod")},
    {_T("MkdswwrtCommand"), IDS_CMD_MKDSWWRITABLE, _T("MkdswwrtCommandMethod")}

};

#define dimof(a) (sizeof(a)/sizeof(a[0]))

// Dieser Code wird beim ersten Laden des Add-Ins und beim Starten der Anwendung aufgerufen
//  jeder nachfolgenden Developer Studio-Sitzung
STDMETHODIMP CDSAddIn::OnConnection(IApplication* pApp, VARIANT_BOOL bFirstTime,
		long dwCookie, VARIANT_BOOL* OnConnection)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	// An uns bergebene Info speichern
	IApplication* pApplication = NULL;
	if (FAILED(pApp->QueryInterface(IID_IApplication, (void**) &pApplication))
		|| pApplication == NULL)
	{
		*OnConnection = VARIANT_FALSE;
		return S_OK;
	}

	m_dwCookie = dwCookie;

	// Befehlsverteilung erzeugen, Rckmeldung an DevStudio
	CCommandsObj::CreateInstance(&m_pCommands);
	m_pCommands->AddRef();

	// Das obige QueryInterface hat AddRef auf das Objekt Application angewendet. Es
	//  wird im Destruktor von CCommand freigegeben.
	m_pCommands->SetApplicationObject(pApplication);

	// (siehe Definition von VERIFY_OK in stdafx.h)

	VERIFY_OK(pApplication->SetAddInInfo((long) AfxGetInstanceHandle(),
		(LPDISPATCH) m_pCommands, IDR_TOOLBAR_MEDIUM, IDR_TOOLBAR_LARGE, m_dwCookie));

	// DevStudio ber die implementierten Befehle informieren

	// ZU ERLEDIGEN: Ersetzen Sie den nachstehenden Aufruf von AddCommand durch je einen Aufruf,
	//  fr jeden durch Ihr Add-In hinzugefgten Befehl.

	// Der Befehlsname sollte nicht in andere Sprachen bersetzt werden. Die
	//  QuickInfo, Befehlsbeschreibung und andere Zeichenfolgen, die mit diesem Befehl verknpft sind,
	//  werden in der String-Tabelle (IDS_CMD_STRING) gespeichert und sollten
	//  lokalisiert werden.

    size_t stIdx = 0;

    for(;stIdx<dimof(g_cmdSet);stIdx++)
    {
        LPCTSTR szCommand = g_cmdSet[stIdx].szCmdName;
	    VARIANT_BOOL bRet;
	    CString strCmdString;
	    strCmdString.LoadString(g_cmdSet[stIdx].uiCmdLocalizedId);
	    strCmdString = szCommand + strCmdString;
	    CComBSTR bszCmdString(strCmdString);
	    CComBSTR bszMethod(g_cmdSet[stIdx].szCmdMethod);
	    CComBSTR bszCmdName(szCommand);
	    VERIFY_OK(pApplication->AddCommand(bszCmdString, bszMethod, stIdx, m_dwCookie, &bRet));
	    if (bRet == VARIANT_FALSE)
	    {
		    // AddCommand fehlgeschlagen, da ein Befehl dieses Namens bereits
		    //  existiert. Sie knnen versuchen, Ihren Befehl unter einem anderen Namen hinzuzufgen.
		    //  Oder das Laden schlgt fehl, wie wir hier demonstrieren werden.
		    *OnConnection = VARIANT_FALSE;
		    return S_OK;
	    }

        // Fgen Sie die Schaltflchen fr die Symbolleiste nur beim ersten Laden des Add-Ins hinzu.
	    //  Die Schaltflchen der Symbolleiste werden von Developer Studio automatisch
	    //  zwischen den Sitzungen gespeichert, deshalb sollten Sie die Schaltflchen der
	    //  Symbolleisten-Schaltflche nur einmal hinzufgen.
	    if (bFirstTime == VARIANT_TRUE)
	    {
		    VERIFY_OK(pApplication->
			    AddCommandBarButton(dsGlyph, bszCmdName, m_dwCookie));
	    }
    }

	*OnConnection = VARIANT_TRUE;
	return S_OK;
}

// Dieser Code wird bei Shut-Down-Vorgngen und beim Entfernen des Add-Ins aus dem Speicher aufgerufen
STDMETHODIMP CDSAddIn::OnDisconnection(VARIANT_BOOL /*bLastTime*/)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	m_pCommands->UnadviseFromEvents();
	m_pCommands->Release();
	m_pCommands = NULL;

	// ZU ERLEDIGEN: Hier alle anfallenden Bereinigungsarbeiten durchfhren

	return S_OK;
}
